\name{base64}
\alias{base64}
\alias{base64Encode}
\alias{base64Decode}
\title{Encode/Decode base64 content}
\description{
 These functions encode and decode strings using base64 representations.
 \code{base64} can be used as  a single entry point with an argument to 
 encode or decode. The other two functions perform the specific action.
}
\usage{
base64(txt, encode = !inherits(txt, "base64"), mode = "character")
}
\arguments{
  \item{txt}{character string to encode or decode}
  \item{encode}{logical value indicating whether the desired action is to encode or decode the object.
   If \code{txt} has (S3) class \code{base64}, the default is to decode this.
  }
  \item{mode}{a character string giving either }
}
\details{
  This calls the routines in libcurl. These are not
declared in the curl header files. So the support
may need to be handled carefully on some platforms,
e.g. Microsoft Windows.
}
\value{
 If encode is \code{TRUE}, a character vector
with a class named \code{base64}.
 If decode is \code{TRUE}, a simple string.
}
\references{
 \url{http://curl.haxx.se}{libcurl}
 \url{http://en.wikipedia.org/wiki/Base64}{Wikipedia's explanation of base 64 encoding}
}
\author{Duncan Temple Lang}
\note{
This is currently not vectorized.

We might extend this to work with raw objects.
}
\seealso{
}
\examples{
    # encode and then decode a simple string.
  x = base64("Some simple text for base 64 to handle")
  base64(x)

    # encode to a raw vector
  x = base64("Simple text", TRUE, "raw")

    # decode to a character string.
  ans = base64Decode(x)
    # decoded to a raw format.
  ans = base64Decode(x, "raw")
}
\keyword{IO }
\keyword{programming}
